@@ -9,8 +9,9 @@ import z3c.form.widget
 from z3c.form.interfaces import DISPLAY_MODE, INPUT_MODE, NOVALUE
 from zope.app.pagetemplate import viewpagetemplatefile
 
+from plone.z3cform.widget import singlecheckboxwidget_factory
 from plone.z3cform import MessageFactory as _
-from plone.z3cform.widget import SingleCheckboxWidget
+
 
 class ICrudForm(interface.Interface):
 
@@ -126,16 +127,11 @@ class EditSubForm(form.EditForm):
         return self.content
 
     def _select_field(self):
-        @zope.interface.implementer(z3c.form.interfaces.IDataConverter)
-        def select_widget_factory(field, request):
-            widget = SingleCheckboxWidget(request)
-            return z3c.form.widget.FieldWidget(field, widget)
-
         select_field = field.Field(
             zope.schema.Bool(__name__='select',
                              required=False,
                              title=_(u'select')))
-        select_field.widgetFactory[INPUT_MODE] = select_widget_factory
+        select_field.widgetFactory[INPUT_MODE] = singlecheckboxwidget_factory
         return select_field
         
     # XXX: The three following methods, 'getCombinedWidgets',
@@ -149,17 +145,11 @@ class EditSubForm(form.EditForm):
         for name, widget in list(widgets):
             if widget.mode == INPUT_MODE:
                 view_widget = self.widgets.get('view_%s' % name)
-                if ('title' == name):
-                    
-                    combined.append((widget, ))
-                    combined.append((view_widget, ))
+                if view_widget is not None:
+                    combined.append((widget, view_widget))
                     seen.add(view_widget)
                 else:
-                    if ('title' != name) and (view_widget is not None):
-                        combined.append((widget, view_widget))
-                        seen.add(view_widget)
-                    else:
-                        combined.append((widget,))
+                    combined.append((widget,))
             else:
                 if widget not in seen:
                     combined.append((widget,))
@@ -180,8 +170,12 @@ class EditSubForm(form.EditForm):
 class EditForm(form.Form):
     label = _(u"Edit")
     template = viewpagetemplatefile.ViewPageTemplateFile('crud-table.pt')
-    prefix = 'crud-edit.'
         
+    @property
+    def prefix(self):
+        parent_prefix = getattr(self.context, 'prefix', '')
+        return 'crud-edit.' + parent_prefix
+
     def update(self):
         self._update_subforms()
         super(EditForm, self).update()
@@ -265,10 +259,14 @@ class EditForm(form.Form):
 class AddForm(form.Form):
     label = _(u"Add")
     template = viewpagetemplatefile.ViewPageTemplateFile('../form.pt')
-    prefix = 'crud-add.'
     ignoreContext = True
     ignoreRequest = True
 
+    @property
+    def prefix(self):
+        parent_prefix = getattr(self.context, 'prefix', '')
+        return 'crud-add.' + parent_prefix
+
     @property
     def fields(self):
         return field.Fields(self.context.add_schema)
